@@ -59,9 +59,27 @@ color specified using the option `hl-todo-keyword-faces' as
 foreground color."
   :group 'hl-todo)
 
-(defcustom hl-todo-activate-in-modes '(prog-mode)
-  "Major modes in which `hl-todo-mode' should be activated.
-This is used by `global-hl-todo-mode'."
+(defcustom hl-todo-activate-in-modes '(prog-mode text-mode)
+  "Major-modes in which `hl-todo-mode' should be activated.
+
+This is used by `global-hl-todo-mode', which activates
+`hl-todo-mode' in all buffers whose major-mode derived from one
+of the modes listed here.
+
+Even though `org-mode' indirectly derives from `text-mode' this
+mode is never activated in `org-mode' buffers because that mode
+provides its own TODO keyword handling."
+  :package-version '(hl-todo . "2.1.0")
+  :group 'hl-todo
+  :type '(repeat function))
+
+(defcustom hl-todo-text-modes '(text-mode)
+  "Major-modes that are considered text-modes.
+
+In buffers whose major-mode derives from one of the modes listed
+here TODO keywords are always highlighted even if they are not
+located inside a string."
+  :package-version '(hl-todo . "2.1.0")
   :group 'hl-todo
   :type '(repeat function))
 
@@ -78,11 +96,13 @@ This is used by `global-hl-todo-mode'."
     ("NOTE"   . "#d0bf8f")
     ("KLUDGE" . "#d0bf8f")
     ("HACK"   . "#d0bf8f")
+    ("TEMP"   . "#d0bf8f")
     ("FIXME"  . "#cc9393")
     ("XXX"    . "#cc9393")
     ("XXXX"   . "#cc9393")
     ("???"    . "#cc9393"))
   "Faces used to highlight specific TODO keywords."
+  :package-version '(hl-todo . "2.0.0")
   :group 'hl-todo
   :type '(repeat (cons (string :tag "Keyword")
                        (choice :tag "Face   "
@@ -90,10 +110,15 @@ This is used by `global-hl-todo-mode'."
                                (sexp :tag "Face")))))
 
 (defcustom hl-todo-highlight-punctuation ""
-  "String of punctuation characters to highlight after keywords.
+  "String of characters to highlight after keywords.
+
 Each of the characters appearing in this string is highlighted
 using the same face as the preceeding keyword when it directly
-follows the keyword."
+follows the keyword.
+
+Characters whose syntax class is `w' (which means word),
+including alphanumeric characters, cannot be used here."
+  :package-version '(hl-todo . "2.0.0")
   :group 'hl-todo
   :type 'string)
 
@@ -102,19 +127,32 @@ follows the keyword."
 
 (defun hl-todo--setup ()
   (setq hl-todo--regexp
-        (concat "\\_<\\("
+        (concat "\\(\\<"
                 (regexp-opt (mapcar #'car hl-todo-keyword-faces) t)
+                "\\>"
                 (and (not (equal hl-todo-highlight-punctuation ""))
-                     (concat "[" hl-todo-highlight-punctuation "]?"))
-                "\\)[[({:;.,?!]?\\_>"))
+                     (concat "[" hl-todo-highlight-punctuation "]*"))
+                "\\)"))
   (setq hl-todo--keywords
-        `(((lambda (limit)
-             (let (case-fold-search)
-               (and (re-search-forward hl-todo--regexp limit t)
-                    (nth 8 (syntax-ppss))))) ; inside comment or string
+        `(((lambda (bound) (hl-todo--search nil bound))
            (1 (hl-todo--get-face) t t))))
   (font-lock-add-keywords nil hl-todo--keywords t))
 
+(defvar hl-todo--syntax-table (copy-syntax-table text-mode-syntax-table))
+
+(defun hl-todo--search (&optional regexp bound backward)
+  (unless regexp
+    (setq regexp hl-todo--regexp))
+  (and (let ((case-fold-search nil))
+         (with-syntax-table hl-todo--syntax-table
+           (funcall (if backward #'re-search-backward #'re-search-forward)
+                    regexp bound t)))
+       (or (apply #'derived-mode-p hl-todo-text-modes)
+           (nth 8 (syntax-ppss)) ; inside comment or string
+           (and (or (not bound)
+                    (funcall (if backward #'< #'>) bound (point)))
+                (hl-todo--search regexp bound backward)))))
+
 (defun hl-todo--get-face ()
   (let ((face (cdr (assoc (match-string 2) hl-todo-keyword-faces))))
     (if (stringp face)
@@ -134,21 +172,19 @@ follows the keyword."
       (hl-todo--setup)
     (font-lock-remove-keywords nil hl-todo--keywords))
   (when font-lock-mode
-    (if (and (fboundp 'font-lock-flush)
-             (fboundp 'font-lock-ensure))
-        (save-restriction
-          (widen)
-          (font-lock-flush)
-          (font-lock-ensure))
-      (with-no-warnings
-        (font-lock-fontify-buffer)))))
+    (save-excursion
+      (goto-char (point-min))
+      (while (hl-todo--search)
+        (save-excursion
+	  (font-lock-fontify-region (match-beginning 0) (match-end 0) nil))))))
 
 ;;;###autoload
 (define-globalized-minor-mode global-hl-todo-mode
   hl-todo-mode hl-todo--turn-on-mode-if-desired)
 
 (defun hl-todo--turn-on-mode-if-desired ()
-  (when (apply #'derived-mode-p hl-todo-activate-in-modes)
+  (when (and (apply #'derived-mode-p hl-todo-activate-in-modes)
+             (not (derived-mode-p 'org-mode)))
     (hl-todo-mode 1)))
 
 ;;;###autoload
@@ -161,10 +197,11 @@ A negative argument means move backward that many keywords."
       (hl-todo-previous (- arg))
     (while (and (> arg 0)
                 (not (eobp))
-                (let ((case-fold-search nil))
-                  (when (looking-at hl-todo--regexp)
+                (progn
+                  (when (let ((case-fold-search nil))
+                          (looking-at hl-todo--regexp))
                     (goto-char (match-end 0)))
-                  (or (re-search-forward hl-todo--regexp nil t)
+                  (or (hl-todo--search)
                       (user-error "No more matches"))))
       (cl-decf arg))))
 
@@ -178,10 +215,9 @@ A negative argument means move forward that many keywords."
       (hl-todo-next (- arg))
     (while (and (> arg 0)
                 (not (bobp))
-                (let ((case-fold-search nil)
-                      (start (point)))
-                  (re-search-backward (concat hl-todo--regexp "\\=") nil t)
-                  (or (re-search-backward hl-todo--regexp nil t)
+                (let ((start (point)))
+                  (hl-todo--search (concat hl-todo--regexp "\\=") nil t)
+                  (or (hl-todo--search nil nil t)
                       (progn (goto-char start)
                              (user-error "No more matches")))))
       (goto-char (match-end 0))
